
const complotDB = require('../database/ComplotDB');
const channelsID = require('../channels_conf.json');
const {Category} = require('../database/Category.js');

const COMMAND_LIST = '!list';
const COMMAND_ADD = '!add';
const COMMAND_REMOVE = '!remove';
const ADMIN_CHAN = channelsID.admin;

class DatabaseCommand{
    constructor(client, baseInfo){
        this.client = client;
        this.baseInfo = baseInfo;
        this.adminChan = this.client.channels.cache.get(ADMIN_CHAN);
    }

    parse(message){
        if(message.channelId === ADMIN_CHAN){
            const content = message.content;
            if(content.startsWith(COMMAND_LIST)){
                this.sendList();
            }else if(content.startsWith(COMMAND_ADD)){
                this.parseAdd(message);
            }else if(content.startsWith(COMMAND_REMOVE)){
                this.parseRemove(message);
            }
        }
    }

    async parseRemove(message) {
        let category, indexes;
        const params = message.content.split(' ');
        if(params.length < 3){
            this.sendHelpRemove(message);
            return;
        }
        try{
            category = params[1].toLowerCase();
            const value = params.slice(2).join(' ');
            if(value.includes(',')){
                indexes = JSON.parse(`[${value}]`);
            }else{
                indexes = [value];
            }
            indexes = this.#toInteger(indexes);
        }catch(e){
            console.log(e);
            this.sendHelpRemove(message);
            return;
        }

        if(indexes.length < 1){
            this.sendHelpRemove(message);
            return;
        }
        const response = await this.remove(category, indexes);
        message.reply(response);
    }


    async parseAdd(message) {
        let category, content;
        const params = message.content.split(' ');
        if(params.length < 3){
            this.sendHelpAdd(message);
            return;
        }
        category = params[1].toLowerCase();
        content = params.slice(2).join(' ');
        
        const reponse = await this.add(category, content);
        message.reply(reponse);
    }

    sendHelpRemove(message) {
        message.reply('Utilisation : !remove <actor, action, reason, source,welcome, sentence> 0,1,2,3,4');
    }

    sendHelpAdd(message) {
        message.reply('Utilisation : !add <actor, action, reason, source, welcome, sentence> <content>');
    }



    wrongCatMessage(category){
        return `'${category}' n'est pas 'actor', 'action', 'reason', 'source', 'welcome' ou 'sentence`;
    }

    async add(category, content) {
        const list = complotDB.get(category);
        if(list === null){
            return wrongCatMessage(category);
        }
        list.push(content);

        await this.baseInfo.updateBase(category);

        complotDB.saveFile();

        return `'${content}' a été ajouté à '${category}'`; 
    }

    async remove(category, indexes) {
        const list = complotDB.get(category);
        if(list === null){
            return wrongCatMessage(category);
        }
        const wrongIndexes = indexes.filter(index => index < 0 || index >= list.length);
        if(wrongIndexes.length > 0) return `${wrongIndexes.join(',')} invalides pour '${category}'`;

        const newList = list.filter((_, index) => !indexes.includes(index));
        const removedItems = list.filter((_, index) => indexes.includes(index)).map((e) => `'${e}'`).join(', ');

        list.splice(0); //Empty
        list.push(...newList);

        await this.baseInfo.updateBase(category);

        await this.sendCategory(category);

        complotDB.saveFile();

        return `Removed : ${removedItems}`;
    }


    #toInteger(stringList) {
        const res = stringList.map((element, index) => {
          const value = parseInt(element, 10);
          if (isNaN(value)) {
            throw new Error(`Not a number: ${element}.`);
          }
          return value;
        });
      
        return res;
      }

    async sendList() {
        const allCats = Object.values(Category);
        for(let cat of allCats) {
            this.sendCategory(cat);
        }
    }

    async sendCategory(category){
        let allMessages = complotDB.categoryToSplitMessage(category).map(el => '```' + el + '```');
        for(let message of allMessages){
            await this.adminChan.send(message);
        }
    }
}

module.exports = DatabaseCommand;