const fs = require('fs');
const {Category} = require('./Category');


class ComplotDB{
    constructor(){
        this.filename = "./database/complot_db.json";
        this.actors = [];
        this.actions = [];
        this.reasons = [];
        this.sentences = [];
        this.sources = [];
        this.welcome = [];
        this.readFile();
    }

    add(category, content){
        this.get(category).push(content);
        this.saveFile();
    }

    get(category){
        switch (category){
            case Category.ACTION : return this.actions;
            case Category.REASON : return this.reasons;
            case Category.SENTENCE : return this.sentences;
            case Category.WELCOME : return this.welcome;
            case Category.SOURCE : return this.sources;
            case Category.ACTOR : return this.actors;
        }

        return null;
    }

    getRandom(category){
        const list = this.get(category);
        return list[Math.floor(Math.random()*list.length)];
    }

    categoryToString(category){
        const list = this.get(category);
        const title = category.charAt(0).toUpperCase() + category.slice(1);
        let string = `======== ${title} =========\n`;
        for(let i = 0; i < list.length; i++){
            string += `${i} - ${list[i]}\n`;
        }
        string += "\n";
        return string;
    }

    categoryToSplitMessage(category) {
        const text = this.categoryToString(category);
        
        let textLines = text.split("\n");
        let splitArray = [];
        let currentTextPart = "";

        for (let line of textLines) {
            if ((currentTextPart + line).length > 1950) {
                splitArray.push(currentTextPart);
                currentTextPart = line;
            } else currentTextPart += ("\n" + line);
        }

        splitArray.push(currentTextPart);
        return splitArray;
    }

    readFile(){
        fs.readFile(this.filename, (err, data)=>{
            if (err){
                console.log("ERROR READ FILE", err);
                throw err;
            }
            const json = JSON.parse(data);
            this.actors = json.actors ? json.actors : [];
            this.reasons = json.reasons ? json.reasons : [];
            this.actions = json.actions ? json.actions : [];
            this.sources = json.sources ? json.sources : [];
            this.sentences = json.sentences ? json.sentences : [];
            this.welcome = json.welcome ? json.welcome : [];
        });
    }

    saveFile(){
        let data = JSON.stringify({
            actors: this.actors, 
            actions: this.actions,
            reasons: this.reasons,
            sources : this.sources,
            sentences : this.sentences,
            welcome: this.welcome,
        });

        fs.writeFile(this.filename, data, (err)=>{
            if(err){
                console.log("SAVE FILE ERROR", err);
            }
        });
    }


}

const instance = new ComplotDB();

module.exports = instance;